{
  This file was automatically generated on 2012-12-20.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletGPS;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_GPS_FUNCTION_GET_COORDINATES = 1;
  BRICKLET_GPS_FUNCTION_GET_STATUS = 2;
  BRICKLET_GPS_FUNCTION_GET_ALTITUDE = 3;
  BRICKLET_GPS_FUNCTION_GET_MOTION = 4;
  BRICKLET_GPS_FUNCTION_GET_DATE_TIME = 5;
  BRICKLET_GPS_FUNCTION_RESTART = 6;
  BRICKLET_GPS_FUNCTION_SET_COORDINATES_CALLBACK_PERIOD = 7;
  BRICKLET_GPS_FUNCTION_GET_COORDINATES_CALLBACK_PERIOD = 8;
  BRICKLET_GPS_FUNCTION_SET_STATUS_CALLBACK_PERIOD = 9;
  BRICKLET_GPS_FUNCTION_GET_STATUS_CALLBACK_PERIOD = 10;
  BRICKLET_GPS_FUNCTION_SET_ALTITUDE_CALLBACK_PERIOD = 11;
  BRICKLET_GPS_FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD = 12;
  BRICKLET_GPS_FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD = 13;
  BRICKLET_GPS_FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD = 14;
  BRICKLET_GPS_FUNCTION_SET_MOTION_CALLBACK_PERIOD = 15;
  BRICKLET_GPS_FUNCTION_GET_MOTION_CALLBACK_PERIOD = 16;
  BRICKLET_GPS_CALLBACK_COORDINATES = 17;
  BRICKLET_GPS_CALLBACK_STATUS = 18;
  BRICKLET_GPS_CALLBACK_ALTITUDE = 19;
  BRICKLET_GPS_CALLBACK_MOTION = 20;
  BRICKLET_GPS_CALLBACK_DATE_TIME = 21;

type
  TBrickletGPSNotifyCoordinates = procedure(const latitude: longword; const ns: char; const longitude: longword; const ew: char; const pdop: word; const hdop: word; const vdop: word; const epe: word) of object;
  TBrickletGPSNotifyStatus = procedure(const fix: byte; const satellitesView: byte; const satellitesUsed: byte) of object;
  TBrickletGPSNotifyAltitude = procedure(const altitude: longword; const geoidalSeparation: longword) of object;
  TBrickletGPSNotifyMotion = procedure(const course: longword; const speed: longword) of object;
  TBrickletGPSNotifyDateTime = procedure(const date: longword; const time: longword) of object;

  /// <summary>
  ///  Device for receiving GPS position
  /// </summary>
  TBrickletGPS = class(TDevice)
  private
    coordinatesCallback: TBrickletGPSNotifyCoordinates;
    statusCallback: TBrickletGPSNotifyStatus;
    altitudeCallback: TBrickletGPSNotifyAltitude;
    motionCallback: TBrickletGPSNotifyMotion;
    dateTimeCallback: TBrickletGPSNotifyDateTime;
  protected
    procedure CallbackWrapperCoordinates(const packet: TByteArray); virtual;
    procedure CallbackWrapperStatus(const packet: TByteArray); virtual;
    procedure CallbackWrapperAltitude(const packet: TByteArray); virtual;
    procedure CallbackWrapperMotion(const packet: TByteArray); virtual;
    procedure CallbackWrapperDateTime(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Returns the GPS coordinates. Latitude and longitude are given in the
    ///  DD.dddddd° format. The parameter ns and ew are the cardinal directions for
    ///  latitude and logitude. Possible values for ns and ew are 'N', 'S', 'E'
    ///  and 'W' (north, south, east and west).
    ///  
    ///  PDOP, HDOP and VDOP are the dilution of precision (DOP) values. They specify
    ///  the additional multiplicative effect of GPS satellite geometry on GPS 
    ///  precision. See 
    ///  `here &lt;http://en.wikipedia.org/wiki/Dilution_of_precision_(GPS)&gt;`__
    ///  for more information. The values are give in hundreths.
    ///  
    ///  EPE is the "Estimated Position Error". The EPE is given in cm. This is not the
    ///  absolute maximum error, it is the error with a specific confidence. See
    ///  `here &lt;http://www.nps.gov/gis/gps/WhatisEPE.html&gt;`__ for more information.
    /// </summary>
    procedure GetCoordinates(out latitude: longword; out ns: char; out longitude: longword; out ew: char; out pdop: word; out hdop: word; out vdop: word; out epe: word); virtual;

    /// <summary>
    ///  Returns the current fix status, the number of satellites that are in view and
    ///  the number of satellites that are currently used.
    ///  
    ///  Possible fix status values can be:
    ///  
    ///  <code>
    ///   "Value", "Description"
    ///  
    ///   "1", "No Fix"
    ///   "2", "2D Fix"
    ///   "3", "3D Fix"
    ///  </code>
    /// </summary>
    procedure GetStatus(out fix: byte; out satellitesView: byte; out satellitesUsed: byte); virtual;

    /// <summary>
    ///  Returns the current altitude and corresponding geoidal separation.
    ///  
    ///  Both values are given in cm.
    /// </summary>
    procedure GetAltitude(out altitude: longword; out geoidalSeparation: longword); virtual;

    /// <summary>
    ///  Returns the current course and speed. Course is given in hundreths degree
    ///  and speed is given in hundreths km/h.
    ///  
    ///  Please note that this only returns useful values if an actual movement
    ///  is present.
    /// </summary>
    procedure GetMotion(out course: longword; out speed: longword); virtual;

    /// <summary>
    ///  Returns the current date and time. The date is
    ///  given in the format ddmmyy and the time is given
    ///  in the format hhmmss.sss
    /// </summary>
    procedure GetDateTime(out date: longword; out time: longword); virtual;

    /// <summary>
    ///  Restarts the GPS Bricklet, the following restart types are available:
    ///  
    ///  <code>
    ///   "Value", "Description"
    ///  
    ///   "0", "Hot start (use all available data in the NV store)"
    ///   "1", "Warm start (don't use ephemeris at restart)"
    ///   "2", "Cold start (don't use time, position, almanacs and ephemeris at restart)"
    ///   "3", "Factory reset (clear all system/user configurations at restart)"
    ///  </code>
    /// </summary>
    procedure Restart(const restartType: byte); virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletGPS.TBrickletGPS.Coordinates"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Coordinates"/> is only triggered if the coordinates changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetCoordinatesCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletGPS.TBrickletGPS.SetCoordinatesCallbackPeriod"/>.
    /// </summary>
    function GetCoordinatesCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletGPS.TBrickletGPS.Status"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Status"/> is only triggered if the status changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetStatusCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletGPS.TBrickletGPS.SetStatusCallbackPeriod"/>.
    /// </summary>
    function GetStatusCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletGPS.TBrickletGPS.Altitude"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Altitude"/> is only triggered if the altitude changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetAltitudeCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletGPS.TBrickletGPS.SetAltitudeCallbackPeriod"/>.
    /// </summary>
    function GetAltitudeCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletGPS.TBrickletGPS.DateTime"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.DateTime"/> is only triggered if the date or time changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetDateTimeCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletGPS.TBrickletGPS.SetDateTimeCallbackPeriod"/>.
    /// </summary>
    function GetDateTimeCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletGPS.TBrickletGPS.Motion"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Motion"/> is only triggered if the motion changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetMotionCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletGPS.TBrickletGPS.SetMotionCallbackPeriod"/>.
    /// </summary>
    function GetMotionCallbackPeriod: longword; virtual;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletGPS.TBrickletGPS.SetCoordinatesCallbackPeriod"/>. The parameters are the same
    ///  as for <see cref="BrickletGPS.TBrickletGPS.GetCoordinates"/>.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Coordinates"/> is only triggered if the coordinates changed since the
    ///  last triggering.
    /// </summary>
    property OnCoordinates: TBrickletGPSNotifyCoordinates read coordinatesCallback write coordinatesCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletGPS.TBrickletGPS.SetStatusCallbackPeriod"/>. The parameters are the same
    ///  as for <see cref="BrickletGPS.TBrickletGPS.GetStatus"/>.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Status"/> is only triggered if the status changed since the
    ///  last triggering.
    /// </summary>
    property OnStatus: TBrickletGPSNotifyStatus read statusCallback write statusCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletGPS.TBrickletGPS.SetAltitudeCallbackPeriod"/>. The parameters are the same
    ///  as for <see cref="BrickletGPS.TBrickletGPS.GetAltitude"/>.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Altitude"/> is only triggered if the altitude changed since the
    ///  last triggering.
    /// </summary>
    property OnAltitude: TBrickletGPSNotifyAltitude read altitudeCallback write altitudeCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletGPS.TBrickletGPS.SetMotionCallbackPeriod"/>. The parameters are the same
    ///  as for <see cref="BrickletGPS.TBrickletGPS.GetMotion"/>.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.Motion"/> is only triggered if the motion changed since the
    ///  last triggering.
    /// </summary>
    property OnMotion: TBrickletGPSNotifyMotion read motionCallback write motionCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletGPS.TBrickletGPS.SetDateTimeCallbackPeriod"/>. The parameters are the same
    ///  as for <see cref="BrickletGPS.TBrickletGPS.GetDateTime"/>.
    ///  
    ///  <see cref="BrickletGPS.TBrickletGPS.DateTime"/> is only triggered if the date or time changed since the
    ///  last triggering.
    /// </summary>
    property OnDateTime: TBrickletGPSNotifyDateTime read dateTimeCallback write dateTimeCallback;
  end;

implementation

constructor TBrickletGPS.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'GPS Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_GPS_CALLBACK_COORDINATES] := {$ifdef FPC}@{$endif}CallbackWrapperCoordinates;
  callbackWrappers[BRICKLET_GPS_CALLBACK_STATUS] := {$ifdef FPC}@{$endif}CallbackWrapperStatus;
  callbackWrappers[BRICKLET_GPS_CALLBACK_ALTITUDE] := {$ifdef FPC}@{$endif}CallbackWrapperAltitude;
  callbackWrappers[BRICKLET_GPS_CALLBACK_MOTION] := {$ifdef FPC}@{$endif}CallbackWrapperMotion;
  callbackWrappers[BRICKLET_GPS_CALLBACK_DATE_TIME] := {$ifdef FPC}@{$endif}CallbackWrapperDateTime;
end;

procedure TBrickletGPS.GetCoordinates(out latitude: longword; out ns: char; out longitude: longword; out ew: char; out pdop: word; out hdop: word; out vdop: word; out epe: word);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_COORDINATES, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_COORDINATES, response);
  latitude := LEConvertUInt32From(4, response);
  ns := LEConvertCharFrom(8, response);
  longitude := LEConvertUInt32From(9, response);
  ew := LEConvertCharFrom(13, response);
  pdop := LEConvertUInt16From(14, response);
  hdop := LEConvertUInt16From(16, response);
  vdop := LEConvertUInt16From(18, response);
  epe := LEConvertUInt16From(20, response);
end;

procedure TBrickletGPS.GetStatus(out fix: byte; out satellitesView: byte; out satellitesUsed: byte);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_STATUS, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_STATUS, response);
  fix := LEConvertUInt8From(4, response);
  satellitesView := LEConvertUInt8From(5, response);
  satellitesUsed := LEConvertUInt8From(6, response);
end;

procedure TBrickletGPS.GetAltitude(out altitude: longword; out geoidalSeparation: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_ALTITUDE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_ALTITUDE, response);
  altitude := LEConvertUInt32From(4, response);
  geoidalSeparation := LEConvertUInt32From(8, response);
end;

procedure TBrickletGPS.GetMotion(out course: longword; out speed: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_MOTION, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_MOTION, response);
  course := LEConvertUInt32From(4, response);
  speed := LEConvertUInt32From(8, response);
end;

procedure TBrickletGPS.GetDateTime(out date: longword; out time: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_DATE_TIME, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_DATE_TIME, response);
  date := LEConvertUInt32From(4, response);
  time := LEConvertUInt32From(8, response);
end;

procedure TBrickletGPS.Restart(const restartType: byte);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_RESTART, 5);
  LEConvertUInt8To(restartType, 4, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletGPS.SetCoordinatesCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_SET_COORDINATES_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletGPS.GetCoordinatesCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_COORDINATES_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_COORDINATES_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletGPS.SetStatusCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_SET_STATUS_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletGPS.GetStatusCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_STATUS_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_STATUS_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletGPS.SetAltitudeCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_SET_ALTITUDE_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletGPS.GetAltitudeCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_ALTITUDE_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletGPS.SetDateTimeCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_SET_DATE_TIME_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletGPS.GetDateTimeCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_DATE_TIME_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletGPS.SetMotionCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_SET_MOTION_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletGPS.GetMotionCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_GPS_FUNCTION_GET_MOTION_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_GPS_FUNCTION_GET_MOTION_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletGPS.CallbackWrapperCoordinates(const packet: TByteArray);
var latitude: longword; ns: char; longitude: longword; ew: char; pdop: word; hdop: word; vdop: word; epe: word;
begin
  if (Assigned(coordinatesCallback)) then begin
    latitude := LEConvertUInt32From(4, packet);
    ns := LEConvertCharFrom(8, packet);
    longitude := LEConvertUInt32From(9, packet);
    ew := LEConvertCharFrom(13, packet);
    pdop := LEConvertUInt16From(14, packet);
    hdop := LEConvertUInt16From(16, packet);
    vdop := LEConvertUInt16From(18, packet);
    epe := LEConvertUInt16From(20, packet);
    coordinatesCallback(latitude, ns, longitude, ew, pdop, hdop, vdop, epe);
  end;
end;

procedure TBrickletGPS.CallbackWrapperStatus(const packet: TByteArray);
var fix: byte; satellitesView: byte; satellitesUsed: byte;
begin
  if (Assigned(statusCallback)) then begin
    fix := LEConvertUInt8From(4, packet);
    satellitesView := LEConvertUInt8From(5, packet);
    satellitesUsed := LEConvertUInt8From(6, packet);
    statusCallback(fix, satellitesView, satellitesUsed);
  end;
end;

procedure TBrickletGPS.CallbackWrapperAltitude(const packet: TByteArray);
var altitude: longword; geoidalSeparation: longword;
begin
  if (Assigned(altitudeCallback)) then begin
    altitude := LEConvertUInt32From(4, packet);
    geoidalSeparation := LEConvertUInt32From(8, packet);
    altitudeCallback(altitude, geoidalSeparation);
  end;
end;

procedure TBrickletGPS.CallbackWrapperMotion(const packet: TByteArray);
var course: longword; speed: longword;
begin
  if (Assigned(motionCallback)) then begin
    course := LEConvertUInt32From(4, packet);
    speed := LEConvertUInt32From(8, packet);
    motionCallback(course, speed);
  end;
end;

procedure TBrickletGPS.CallbackWrapperDateTime(const packet: TByteArray);
var date: longword; time: longword;
begin
  if (Assigned(dateTimeCallback)) then begin
    date := LEConvertUInt32From(4, packet);
    time := LEConvertUInt32From(8, packet);
    dateTimeCallback(date, time);
  end;
end;

end.
