{
  This file was automatically generated on 2012-12-20.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletJoystick;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_JOYSTICK_FUNCTION_GET_POSITION = 1;
  BRICKLET_JOYSTICK_FUNCTION_IS_PRESSED = 2;
  BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE = 3;
  BRICKLET_JOYSTICK_FUNCTION_CALIBRATE = 4;
  BRICKLET_JOYSTICK_FUNCTION_SET_POSITION_CALLBACK_PERIOD = 5;
  BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_PERIOD = 6;
  BRICKLET_JOYSTICK_FUNCTION_SET_ANALOG_VALUE_CALLBACK_PERIOD = 7;
  BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD = 8;
  BRICKLET_JOYSTICK_FUNCTION_SET_POSITION_CALLBACK_THRESHOLD = 9;
  BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_THRESHOLD = 10;
  BRICKLET_JOYSTICK_FUNCTION_SET_ANALOG_VALUE_CALLBACK_THRESHOLD = 11;
  BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD = 12;
  BRICKLET_JOYSTICK_FUNCTION_SET_DEBOUNCE_PERIOD = 13;
  BRICKLET_JOYSTICK_FUNCTION_GET_DEBOUNCE_PERIOD = 14;
  BRICKLET_JOYSTICK_CALLBACK_POSITION = 15;
  BRICKLET_JOYSTICK_CALLBACK_ANALOG_VALUE = 16;
  BRICKLET_JOYSTICK_CALLBACK_POSITION_REACHED = 17;
  BRICKLET_JOYSTICK_CALLBACK_ANALOG_VALUE_REACHED = 18;
  BRICKLET_JOYSTICK_CALLBACK_PRESSED = 19;
  BRICKLET_JOYSTICK_CALLBACK_RELEASED = 20;

type
  TBrickletJoystickNotifyPosition = procedure(const x: smallint; const y: smallint) of object;
  TBrickletJoystickNotifyAnalogValue = procedure(const x: word; const y: word) of object;
  TBrickletJoystickNotifyPositionReached = procedure(const x: smallint; const y: smallint) of object;
  TBrickletJoystickNotifyAnalogValueReached = procedure(const x: word; const y: word) of object;
  TBrickletJoystickNotifyPressed = procedure of object;
  TBrickletJoystickNotifyReleased = procedure of object;

  /// <summary>
  ///  Dual-Axis Joystick with Button
  /// </summary>
  TBrickletJoystick = class(TDevice)
  private
    positionCallback: TBrickletJoystickNotifyPosition;
    analogValueCallback: TBrickletJoystickNotifyAnalogValue;
    positionReachedCallback: TBrickletJoystickNotifyPositionReached;
    analogValueReachedCallback: TBrickletJoystickNotifyAnalogValueReached;
    pressedCallback: TBrickletJoystickNotifyPressed;
    releasedCallback: TBrickletJoystickNotifyReleased;
  protected
    procedure CallbackWrapperPosition(const packet: TByteArray); virtual;
    procedure CallbackWrapperAnalogValue(const packet: TByteArray); virtual;
    procedure CallbackWrapperPositionReached(const packet: TByteArray); virtual;
    procedure CallbackWrapperAnalogValueReached(const packet: TByteArray); virtual;
    procedure CallbackWrapperPressed(const packet: TByteArray); virtual;
    procedure CallbackWrapperReleased(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Returns the position of the Joystick. The value ranges between -100 and
    ///  100 for both axis. The middle position of the joystick is x=0, y=0. The
    ///  returned values are averaged and calibrated (see <see cref="BrickletJoystick.TBrickletJoystick.Calibrate"/>).
    ///  
    ///  If you want to get the position periodically, it is recommended to use the
    ///  callback <see cref="BrickletJoystick.TBrickletJoystick.Position"/> and set the period with 
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackPeriod"/>.
    /// </summary>
    procedure GetPosition(out x: smallint; out y: smallint); virtual;

    /// <summary>
    ///  Returns *true* if the button is pressed and *false* otherwise.
    ///  
    ///  It is recommended to use the <see cref="BrickletJoystick.TBrickletJoystick.Pressed"/> and <see cref="BrickletJoystick.TBrickletJoystick.Released"/> callbacks
    ///  to handle the button.
    /// </summary>
    function IsPressed: boolean; virtual;

    /// <summary>
    ///  Returns the values as read by a 12-bit analog-to-digital converter.
    ///  The values are between 0 and 4095 for both axis.
    ///  
    ///  <note>
    ///   The values returned by <see cref="BrickletJoystick.TBrickletJoystick.GetPosition"/> are averaged over several samples
    ///   to yield less noise, while <see cref="BrickletJoystick.TBrickletJoystick.GetAnalogValue"/> gives back raw
    ///   unfiltered analog values. The only reason to use <see cref="BrickletJoystick.TBrickletJoystick.GetAnalogValue"/> is,
    ///   if you need the full resolution of the analog-to-digital converter.
    ///  </note>
    ///  
    ///  If you want the analog values periodically, it is recommended to use the 
    ///  callback <see cref="BrickletJoystick.TBrickletJoystick.AnalogValue"/> and set the period with 
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackPeriod"/>.
    /// </summary>
    procedure GetAnalogValue(out x: word; out y: word); virtual;

    /// <summary>
    ///  Calibrates the middle position of the Joystick. If your Joystick Bricklet
    ///  does not return x=0 and y=0 in the middle position, call this function
    ///  while the Joystick is standing still in the middle position.
    ///  
    ///  The resulting calibration will be saved on the EEPROM of the Joystick
    ///  Bricklet, thus you only have to calibrate it once.
    /// </summary>
    procedure Calibrate; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletJoystick.TBrickletJoystick.Position"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletJoystick.TBrickletJoystick.Position"/> is only triggered if the position has changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetPositionCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackPeriod"/>.
    /// </summary>
    function GetPositionCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletJoystick.TBrickletJoystick.AnalogValue"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletJoystick.TBrickletJoystick.AnalogValue"/> is only triggered if the analog values have changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetAnalogValueCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackPeriod"/>.
    /// </summary>
    function GetAnalogValueCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the thresholds for the <see cref="BrickletJoystick.TBrickletJoystick.PositionReached"/> callback. 
    ///  
    ///  The following options are possible:
    ///  
    ///  <code>
    ///   "Option", "Description"
    ///  
    ///   "'x'",    "Callback is turned off"
    ///   "'o'",    "Callback is triggered when the position is *outside* the min and max values"
    ///   "'i'",    "Callback is triggered when the position is *inside* the min and max values"
    ///   "'&lt;'",    "Callback is triggered when the position is smaller than the min values (max is ignored)"
    ///   "'&gt;'",    "Callback is triggered when the position is greater than the min values (max is ignored)"
    ///  </code>
    ///  
    ///  The default value is ('x', 0, 0, 0, 0).
    /// </summary>
    procedure SetPositionCallbackThreshold(const option: char; const minX: smallint; const maxX: smallint; const minY: smallint; const maxY: smallint); virtual;

    /// <summary>
    ///  Returns the threshold as set by <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackThreshold"/>.
    /// </summary>
    procedure GetPositionCallbackThreshold(out option: char; out minX: smallint; out maxX: smallint; out minY: smallint; out maxY: smallint); virtual;

    /// <summary>
    ///  Sets the thresholds for the <see cref="BrickletJoystick.TBrickletJoystick.AnalogValueReached"/> callback. 
    ///  
    ///  The following options are possible:
    ///  
    ///  <code>
    ///   "Option", "Description"
    ///  
    ///   "'x'",    "Callback is turned off"
    ///   "'o'",    "Callback is triggered when the analog values are *outside* the min and max values"
    ///   "'i'",    "Callback is triggered when the analog values are *inside* the min and max values"
    ///   "'&lt;'",    "Callback is triggered when the analog values are smaller than the min values (max is ignored)"
    ///   "'&gt;'",    "Callback is triggered when the analog values are greater than the min values (max is ignored)"
    ///  </code>
    ///  
    ///  The default value is ('x', 0, 0, 0, 0).
    /// </summary>
    procedure SetAnalogValueCallbackThreshold(const option: char; const minX: word; const maxX: word; const minY: word; const maxY: word); virtual;

    /// <summary>
    ///  Returns the threshold as set by <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackThreshold"/>.
    /// </summary>
    procedure GetAnalogValueCallbackThreshold(out option: char; out minX: word; out maxX: word; out minY: word; out maxY: word); virtual;

    /// <summary>
    ///  Sets the period in ms with which the threshold callbacks
    ///  
    ///   <see cref="BrickletJoystick.TBrickletJoystick.PositionReached"/>, <see cref="BrickletJoystick.TBrickletJoystick.AnalogValueReached"/>
    ///  
    ///  are triggered, if the thresholds
    ///  
    ///   <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackThreshold"/>, <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackThreshold"/>
    ///  
    ///  keep being reached.
    ///  
    ///  The default value is 100.
    /// </summary>
    procedure SetDebouncePeriod(const debounce: longword); virtual;

    /// <summary>
    ///  Returns the debounce period as set by <see cref="BrickletJoystick.TBrickletJoystick.SetDebouncePeriod"/>.
    /// </summary>
    function GetDebouncePeriod: longword; virtual;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackPeriod"/>. The parameter is the position of the
    ///  Joystick.
    ///  
    ///  <see cref="BrickletJoystick.TBrickletJoystick.Position"/> is only triggered if the position has changed since the
    ///  last triggering.
    /// </summary>
    property OnPosition: TBrickletJoystickNotifyPosition read positionCallback write positionCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackPeriod"/>. The parameters are the analog values
    ///  of the Joystick.
    ///  
    ///  <see cref="BrickletJoystick.TBrickletJoystick.AnalogValue"/> is only triggered if the values have changed since the
    ///  last triggering.
    /// </summary>
    property OnAnalogValue: TBrickletJoystickNotifyAnalogValue read analogValueCallback write analogValueCallback;

    /// <summary>
    ///  This callback is triggered when the threshold as set by
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetPositionCallbackThreshold"/> is reached.
    ///  The parameters are the position of the Joystick.
    ///  
    ///  If the threshold keeps being reached, the callback is triggered periodically
    ///  with the period as set by <see cref="BrickletJoystick.TBrickletJoystick.SetDebouncePeriod"/>.
    /// </summary>
    property OnPositionReached: TBrickletJoystickNotifyPositionReached read positionReachedCallback write positionReachedCallback;

    /// <summary>
    ///  This callback is triggered when the threshold as set by
    ///  <see cref="BrickletJoystick.TBrickletJoystick.SetAnalogValueCallbackThreshold"/> is reached.
    ///  The parameters are the analog values of the Joystick.
    ///  
    ///  If the threshold keeps being reached, the callback is triggered periodically
    ///  with the period as set by <see cref="BrickletJoystick.TBrickletJoystick.SetDebouncePeriod"/>.
    /// </summary>
    property OnAnalogValueReached: TBrickletJoystickNotifyAnalogValueReached read analogValueReachedCallback write analogValueReachedCallback;

    /// <summary>
    ///  This callback is triggered when the button is pressed.
    /// </summary>
    property OnPressed: TBrickletJoystickNotifyPressed read pressedCallback write pressedCallback;

    /// <summary>
    ///  This callback is triggered when the button is released.
    /// </summary>
    property OnReleased: TBrickletJoystickNotifyReleased read releasedCallback write releasedCallback;
  end;

implementation

constructor TBrickletJoystick.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Joystick Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_POSITION] := {$ifdef FPC}@{$endif}CallbackWrapperPosition;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_ANALOG_VALUE] := {$ifdef FPC}@{$endif}CallbackWrapperAnalogValue;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_POSITION_REACHED] := {$ifdef FPC}@{$endif}CallbackWrapperPositionReached;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_ANALOG_VALUE_REACHED] := {$ifdef FPC}@{$endif}CallbackWrapperAnalogValueReached;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_PRESSED] := {$ifdef FPC}@{$endif}CallbackWrapperPressed;
  callbackWrappers[BRICKLET_JOYSTICK_CALLBACK_RELEASED] := {$ifdef FPC}@{$endif}CallbackWrapperReleased;
end;

procedure TBrickletJoystick.GetPosition(out x: smallint; out y: smallint);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION, response);
  x := LEConvertInt16From(4, response);
  y := LEConvertInt16From(6, response);
end;

function TBrickletJoystick.IsPressed: boolean;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_IS_PRESSED, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_IS_PRESSED, response);
  result := LEConvertBooleanFrom(4, response);
end;

procedure TBrickletJoystick.GetAnalogValue(out x: word; out y: word);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE, response);
  x := LEConvertUInt16From(4, response);
  y := LEConvertUInt16From(6, response);
end;

procedure TBrickletJoystick.Calibrate;
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_CALIBRATE, 4);
  SendRequestNoResponse(request);
end;

procedure TBrickletJoystick.SetPositionCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_SET_POSITION_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletJoystick.GetPositionCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletJoystick.SetAnalogValueCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_SET_ANALOG_VALUE_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletJoystick.GetAnalogValueCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletJoystick.SetPositionCallbackThreshold(const option: char; const minX: smallint; const maxX: smallint; const minY: smallint; const maxY: smallint);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_SET_POSITION_CALLBACK_THRESHOLD, 13);
  LEConvertCharTo(option, 4, request);
  LEConvertInt16To(minX, 5, request);
  LEConvertInt16To(maxX, 7, request);
  LEConvertInt16To(minY, 9, request);
  LEConvertInt16To(maxY, 11, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletJoystick.GetPositionCallbackThreshold(out option: char; out minX: smallint; out maxX: smallint; out minY: smallint; out maxY: smallint);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_THRESHOLD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_POSITION_CALLBACK_THRESHOLD, response);
  option := LEConvertCharFrom(4, response);
  minX := LEConvertInt16From(5, response);
  maxX := LEConvertInt16From(7, response);
  minY := LEConvertInt16From(9, response);
  maxY := LEConvertInt16From(11, response);
end;

procedure TBrickletJoystick.SetAnalogValueCallbackThreshold(const option: char; const minX: word; const maxX: word; const minY: word; const maxY: word);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_SET_ANALOG_VALUE_CALLBACK_THRESHOLD, 13);
  LEConvertCharTo(option, 4, request);
  LEConvertUInt16To(minX, 5, request);
  LEConvertUInt16To(maxX, 7, request);
  LEConvertUInt16To(minY, 9, request);
  LEConvertUInt16To(maxY, 11, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletJoystick.GetAnalogValueCallbackThreshold(out option: char; out minX: word; out maxX: word; out minY: word; out maxY: word);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD, response);
  option := LEConvertCharFrom(4, response);
  minX := LEConvertUInt16From(5, response);
  maxX := LEConvertUInt16From(7, response);
  minY := LEConvertUInt16From(9, response);
  maxY := LEConvertUInt16From(11, response);
end;

procedure TBrickletJoystick.SetDebouncePeriod(const debounce: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_SET_DEBOUNCE_PERIOD, 8);
  LEConvertUInt32To(debounce, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletJoystick.GetDebouncePeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_JOYSTICK_FUNCTION_GET_DEBOUNCE_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_JOYSTICK_FUNCTION_GET_DEBOUNCE_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletJoystick.CallbackWrapperPosition(const packet: TByteArray);
var x: smallint; y: smallint;
begin
  if (Assigned(positionCallback)) then begin
    x := LEConvertInt16From(4, packet);
    y := LEConvertInt16From(6, packet);
    positionCallback(x, y);
  end;
end;

procedure TBrickletJoystick.CallbackWrapperAnalogValue(const packet: TByteArray);
var x: word; y: word;
begin
  if (Assigned(analogValueCallback)) then begin
    x := LEConvertUInt16From(4, packet);
    y := LEConvertUInt16From(6, packet);
    analogValueCallback(x, y);
  end;
end;

procedure TBrickletJoystick.CallbackWrapperPositionReached(const packet: TByteArray);
var x: smallint; y: smallint;
begin
  if (Assigned(positionReachedCallback)) then begin
    x := LEConvertInt16From(4, packet);
    y := LEConvertInt16From(6, packet);
    positionReachedCallback(x, y);
  end;
end;

procedure TBrickletJoystick.CallbackWrapperAnalogValueReached(const packet: TByteArray);
var x: word; y: word;
begin
  if (Assigned(analogValueReachedCallback)) then begin
    x := LEConvertUInt16From(4, packet);
    y := LEConvertUInt16From(6, packet);
    analogValueReachedCallback(x, y);
  end;
end;

procedure TBrickletJoystick.CallbackWrapperPressed(const packet: TByteArray);
begin
  Assert(packet <> nil); { Avoid 'Parameter not used' warning }
  if (Assigned(pressedCallback)) then begin
    pressedCallback();
  end;
end;

procedure TBrickletJoystick.CallbackWrapperReleased(const packet: TByteArray);
begin
  Assert(packet <> nil); { Avoid 'Parameter not used' warning }
  if (Assigned(releasedCallback)) then begin
    releasedCallback();
  end;
end;

end.
