{
  This file was automatically generated on 2012-12-20.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletLCD20x4;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_LCD_20X4_FUNCTION_WRITE_LINE = 1;
  BRICKLET_LCD_20X4_FUNCTION_CLEAR_DISPLAY = 2;
  BRICKLET_LCD_20X4_FUNCTION_BACKLIGHT_ON = 3;
  BRICKLET_LCD_20X4_FUNCTION_BACKLIGHT_OFF = 4;
  BRICKLET_LCD_20X4_FUNCTION_IS_BACKLIGHT_ON = 5;
  BRICKLET_LCD_20X4_FUNCTION_SET_CONFIG = 6;
  BRICKLET_LCD_20X4_FUNCTION_GET_CONFIG = 7;
  BRICKLET_LCD_20X4_FUNCTION_IS_BUTTON_PRESSED = 8;
  BRICKLET_LCD_20X4_CALLBACK_BUTTON_PRESSED = 9;
  BRICKLET_LCD_20X4_CALLBACK_BUTTON_RELEASED = 10;

type
  TBrickletLCD20x4NotifyButtonPressed = procedure(const button: byte) of object;
  TBrickletLCD20x4NotifyButtonReleased = procedure(const button: byte) of object;

  /// <summary>
  ///  Device for controlling a LCD with 4 lines a 20 characters
  /// </summary>
  TBrickletLCD20x4 = class(TDevice)
  private
    buttonPressedCallback: TBrickletLCD20x4NotifyButtonPressed;
    buttonReleasedCallback: TBrickletLCD20x4NotifyButtonReleased;
  protected
    procedure CallbackWrapperButtonPressed(const packet: TByteArray); virtual;
    procedure CallbackWrapperButtonReleased(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Writes text to a specific line (0 to 3) with a specific position 
    ///  (0 to 19). The text can have a maximum of 20 characters.
    ///  
    ///  For example: (0, 7, "Hello") will write *Hello* in the middle of the
    ///  first line of the display.
    ///  
    ///  The display uses a special charset that includes all ASCII characters except
    ///  backslash and tilde. The LCD charset also includes several other non-ASCII characters, see
    ///  the `charset specification &lt;https://github.com/Tinkerforge/lcd-20x4-bricklet/raw/master/datasheets/standard_charset.pdf&gt;`__
    ///  for details. The Unicode example above shows how to specify non-ASCII characters
    ///  and how to translate from Unicode to the LCD charset.
    /// </summary>
    procedure WriteLine(const line: byte; const position: byte; const text: string); virtual;

    /// <summary>
    ///  Deletes all characters from the display.
    /// </summary>
    procedure ClearDisplay; virtual;

    /// <summary>
    ///  Turns the backlight on.
    /// </summary>
    procedure BacklightOn; virtual;

    /// <summary>
    ///  Turns the backlight off.
    /// </summary>
    procedure BacklightOff; virtual;

    /// <summary>
    ///  Returns *true* if the backlight is on and *false* otherwise.
    /// </summary>
    function IsBacklightOn: boolean; virtual;

    /// <summary>
    ///  Configures if the cursor (shown as "_") should be visible and if it
    ///  should be blinking (shown as a blinking block). The cursor position
    ///  is one character behind the the last text written with 
    ///  <see cref="BrickletLCD20x4.TBrickletLCD20x4.WriteLine"/>.
    ///  
    ///  The default is (false, false).
    /// </summary>
    procedure SetConfig(const cursor: boolean; const blinking: boolean); virtual;

    /// <summary>
    ///  Returns the configuration as set by <see cref="BrickletLCD20x4.TBrickletLCD20x4.SetConfig"/>.
    /// </summary>
    procedure GetConfig(out cursor: boolean; out blinking: boolean); virtual;

    /// <summary>
    ///  Returns *true* if the button (0 to 2) is pressed. If you want to react
    ///  on button presses and releases it is recommended to use the
    ///  <see cref="BrickletLCD20x4.TBrickletLCD20x4.ButtonPressed"/> and <see cref="BrickletLCD20x4.TBrickletLCD20x4.ButtonReleased"/> callbacks.
    /// </summary>
    function IsButtonPressed(const button: byte): boolean; virtual;

    /// <summary>
    ///  This callback is triggered when a button is pressed. The parameter is
    ///  the number of the button (0 to 2).
    /// </summary>
    property OnButtonPressed: TBrickletLCD20x4NotifyButtonPressed read buttonPressedCallback write buttonPressedCallback;

    /// <summary>
    ///  This callback is triggered when a button is released. The parameter is
    ///  the number of the button (0 to 2).
    /// </summary>
    property OnButtonReleased: TBrickletLCD20x4NotifyButtonReleased read buttonReleasedCallback write buttonReleasedCallback;
  end;

implementation

constructor TBrickletLCD20x4.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'LCD 20x4 Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_LCD_20X4_CALLBACK_BUTTON_PRESSED] := {$ifdef FPC}@{$endif}CallbackWrapperButtonPressed;
  callbackWrappers[BRICKLET_LCD_20X4_CALLBACK_BUTTON_RELEASED] := {$ifdef FPC}@{$endif}CallbackWrapperButtonReleased;
end;

procedure TBrickletLCD20x4.WriteLine(const line: byte; const position: byte; const text: string);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_WRITE_LINE, 26);
  LEConvertUInt8To(line, 4, request);
  LEConvertUInt8To(position, 5, request);
  LEConvertStringTo(text, 6, 20, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletLCD20x4.ClearDisplay;
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_CLEAR_DISPLAY, 4);
  SendRequestNoResponse(request);
end;

procedure TBrickletLCD20x4.BacklightOn;
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_BACKLIGHT_ON, 4);
  SendRequestNoResponse(request);
end;

procedure TBrickletLCD20x4.BacklightOff;
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_BACKLIGHT_OFF, 4);
  SendRequestNoResponse(request);
end;

function TBrickletLCD20x4.IsBacklightOn: boolean;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_IS_BACKLIGHT_ON, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_LCD_20X4_FUNCTION_IS_BACKLIGHT_ON, response);
  result := LEConvertBooleanFrom(4, response);
end;

procedure TBrickletLCD20x4.SetConfig(const cursor: boolean; const blinking: boolean);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_SET_CONFIG, 6);
  LEConvertBooleanTo(cursor, 4, request);
  LEConvertBooleanTo(blinking, 5, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletLCD20x4.GetConfig(out cursor: boolean; out blinking: boolean);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_GET_CONFIG, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_LCD_20X4_FUNCTION_GET_CONFIG, response);
  cursor := LEConvertBooleanFrom(4, response);
  blinking := LEConvertBooleanFrom(5, response);
end;

function TBrickletLCD20x4.IsButtonPressed(const button: byte): boolean;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_LCD_20X4_FUNCTION_IS_BUTTON_PRESSED, 5);
  LEConvertUInt8To(button, 4, request);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_LCD_20X4_FUNCTION_IS_BUTTON_PRESSED, response);
  result := LEConvertBooleanFrom(4, response);
end;

procedure TBrickletLCD20x4.CallbackWrapperButtonPressed(const packet: TByteArray);
var button: byte;
begin
  if (Assigned(buttonPressedCallback)) then begin
    button := LEConvertUInt8From(4, packet);
    buttonPressedCallback(button);
  end;
end;

procedure TBrickletLCD20x4.CallbackWrapperButtonReleased(const packet: TByteArray);
var button: byte;
begin
  if (Assigned(buttonReleasedCallback)) then begin
    button := LEConvertUInt8From(4, packet);
    buttonReleasedCallback(button);
  end;
end;

end.
